// Written by Luodiping 2008
// Use & modify as you want & need, and leave those 3 lines.
// http://duceland.com

#include <string.h>
#include "LWideString.h"

#ifdef _DS_
#include "GlobalFunc.h"
#define NEW_WCHAR(len) (wchar_t *)g_RuntimeVars.vm->ReallocStringData(this,len)
#define DEL_WCHAR(text) g_RuntimeVars.vm->Unalloc(NULL,text)
#else
#define NEW_WCHAR(len) (wchar_t *)MemAlloc(sizeof(wchar_t)*len)
#define DEL_WCHAR(text) MemFree(text)
#endif

LWideString::LWideString():AllocSize(0),TextLength(0),Text(NULL)
{
}
LWideString::LWideString(const LWideString &rhs):AllocSize(0),TextLength(0),Text(NULL)
{
  operator=(rhs);
}
LWideString::LWideString(const wchar_t *rhs):AllocSize(0),TextLength(0),Text(NULL)
{
  operator=(rhs);
}
LWideString::~LWideString()
{
  DEL_WCHAR(Text);
}
wchar_t *LWideString::GetText()const
{
  return Text;
}
int LWideString::GetLength()const
{
  return TextLength;
}
void LWideString::ResetSize(int len)
{                                                                          
  if(len==0)
  {
    if(Text!=NULL)
      Text[0]=L'\0';
    TextLength=0;
    return;
  }
  if(AllocSize<len)
  {
    DEL_WCHAR(Text);
    AllocSize=((len+8)/8+1)*8;
    Text=NEW_WCHAR(AllocSize+8);
    memset(Text,0,sizeof(wchar_t)*AllocSize);
    TextLength=0;
    return;
  }
}
LWideString &LWideString::operator=(const LWideString &rhs)
{
  *this=rhs.GetText();
  return *this;
}
LWideString &LWideString::operator=(unsigned char *rhs)
{
  int len;
  if(rhs==NULL)
    len=0;
  else
    len=strlen((char *)rhs);
  ResetSize(len);
  if(len==0)
    return *this;
  len=MultiBytesToWides(Text,(char *)rhs,len);
  if(len==0)
  {
    TextLength=0;
  }
  else
  {
    TextLength=len;
  }
  Text[TextLength]=L'\0';
  return *this;
}
LWideString &LWideString::operator=(const char *rhs)
{
  int len;
  if(rhs==NULL)
    len=0;
  else
    len=strlen(rhs);
  ResetSize(len);
  if(len==0)
    return *this;
  len=MultiBytesToWides(Text,rhs,len);
  if(len==0)
  {
    TextLength=0;
  }
  else
  {
    TextLength=len;
  }
  Text[TextLength]=L'\0';
  return *this;
}
LWideString &LWideString::operator=(const wchar_t *rhs)
{
  int len;
  if(rhs==NULL)
    len=0;
  else
    len=wcslen(rhs);
  ResetSize(len);
  if(len==0)
    return *this;
  memcpy(Text,rhs,sizeof(wchar_t)*len);
  TextLength=len;
  Text[TextLength]=L'\0';
  return *this;
}
bool LWideString::operator==(LWideString &rhs)
{
  return Compare(rhs)==0;
}
bool LWideString::operator==(unsigned char *rhs)
{
  return Compare(rhs)==0;
}
bool LWideString::operator==(wchar_t *rhs)
{
  return Compare(rhs)==0;
}
bool LWideString::operator!=(LWideString &rhs)
{
  return !(Compare(rhs)==0);
}
bool LWideString::operator!=(unsigned char *rhs)
{
  return !(Compare(rhs)==0);
}
bool LWideString::operator!=(wchar_t *rhs)
{
  return !(Compare(rhs)==0);
}
bool LWideString::operator<(const LWideString &rhs) const
{
  return (Compare(rhs)<0);
}
void LWideString::Clear()
{
  if(Text!=NULL)
    memset(Text,0,sizeof(wchar_t)*AllocSize);
  TextLength=0;
}
void LWideString::SetLength(int len)
{
  if(len==0)
  {
    if(Text!=NULL)
      memset(Text,0,sizeof(wchar_t)*AllocSize);
  }
  else
  if(Text==NULL)
  {
    AllocSize=((len+128)/128+1)*128;
    Text=NEW_WCHAR(AllocSize);
    memset(Text,0,sizeof(wchar_t)*AllocSize);
  }else
  if(AllocSize<=len)
  {
    wchar_t *text;
    AllocSize=((len+128)/128+1)*128;
    text=NEW_WCHAR(AllocSize);
    if(TextLength>0)
      memcpy(text,Text,sizeof(wchar_t)*TextLength);
    DEL_WCHAR(Text);
    Text=text;
  }
  if(len>0 && len<TextLength)
  {
    TextLength=len;
    Text[TextLength]=L'\0';
  }
}
LWideString &LWideString::Add(wchar_t c)
{
  SetLength(TextLength+2);
  Text[TextLength++]=c;
  Text[TextLength]=L'\0';
  return *this;
}
LWideString &LWideString::Add(wchar_t *s)
{
  if(s==NULL)
    return *this;

  int len=wcslen(s);
  SetLength(TextLength+len);
  memcpy(Text+TextLength,s,sizeof(wchar_t)*len);
  TextLength+=len;
  Text[TextLength]=L'\0';
  return *this;
}
LWideString &LWideString::Add(LWideString &s)
{
  int len=s.GetLength();
  if(len==0)
    return *this;
  SetLength(TextLength+len);
  memcpy(Text+TextLength,s.Text,sizeof(wchar_t)*len);
  TextLength+=len;
  Text[TextLength]=L'\0';
  return *this;
}
wchar_t &LWideString::operator[](int index)
{
  if(index<TextLength)
    return Text[index];
  static dchar ret[1]={0};
  return ret[0];
}
wchar_t LWideString::Get(int index)
{
  if(index<TextLength)
    return Text[index];
  return L'\0';
}
LWideString &LWideString::Copy(wchar_t *rhs,int wchar_len)
{
  if(wchar_len==0 || rhs==NULL)
  {
    if(Text!=NULL)
      Text[0]=L'\0';
    TextLength=0;
    return *this;
  }

  ResetSize(wchar_len);
  memcpy(Text,rhs,sizeof(wchar_t)*wchar_len);
  TextLength=wchar_len;
  Text[TextLength]=L'\0';

  return *this;
}
LWideString &LWideString::Copy(unsigned char *rhs,int char_len)
{
  if(char_len==0 || rhs==NULL)
  {
    if(Text!=NULL)
      Text[0]=L'\0';
    TextLength=0;
    return *this;
  }

  ResetSize(char_len);

  MultiBytesToWides(Text,(char *)rhs,char_len);
  TextLength=char_len;
  Text[TextLength]=L'\0';

  return *this;
}
LWideString &LWideString::Insert(int pos,const LWideString &source)
{
  if(source.GetLength()==0)
  {
    //do noting
  }
  else
  if(TextLength==0)
  {
    return Copy(source.GetText(),source.GetLength());
  }
  else
  {
    if(pos>TextLength)
      pos=TextLength;
    SetLength(TextLength+source.GetLength());
    memcpy(Text+TextLength,source.GetText(),sizeof(wchar_t)*source.GetLength());
    TextLength+=source.GetLength();
    Text[TextLength]=L'\0';
  }
  return *this;
}
LWideString &LWideString::Append(const LWideString &source)
{
  return Insert(TextLength,source);
}
LWideString &LWideString::Insert(int pos,wchar_t *source)
{
  if(TextLength==0)
  {
    return operator=(source);
  }
  else
  {
    if(pos>TextLength)
      pos=TextLength;
    int len=(int)wcslen(source);
    SetLength(TextLength+len);
    memcpy(Text+TextLength,source,sizeof(wchar_t)*len);
    TextLength+=len;
    Text[TextLength]=L'\0';
  }
  return *this;
}
LWideString &LWideString::Append(wchar_t *source)
{
  return Insert(TextLength,source);
}
LWideString &LWideString::operator+=(LWideString &source)
{
  return Append(source);
}
LWideString &LWideString::operator+=(wchar_t *source)
{
  return Append(source);
}
LWideString &LWideString::operator+=(wchar_t c)
{
  wchar_t s[2];
  s[0]=c;
  s[1]=0;
  return Append(s);
}

LWideString &LWideString::Replace(int pos,int replaced_len,wchar_t *source)
{
  if(TextLength==0)
  {
    return operator=(source);
  }
  else
  if(TextLength<=pos)
  {
    return Append(source);
  }
  else
  if(TextLength<=pos+replaced_len)
  {
    TextLength=pos;
    return Append(source);
  }
  else
  {
    int source_len=wcslen(source);
    wchar_t *new_p,*old_p,*end_p;

    if(replaced_len>=source_len)
    {
      new_p=Text+pos+source_len;
      old_p=Text+pos+replaced_len;
      end_p=Text+TextLength;
      while(old_p<end_p)
        *new_p++=*old_p++;

      memcpy(Text+pos,source,sizeof(wchar_t)*source_len);
      TextLength-=replaced_len-source_len;
    }
    else
    {
      wchar_t *buff;
      if(AllocSize<TextLength+source_len)
      {
        AllocSize=((AllocSize+(source_len-replaced_len)+128)/128+1)*128;
        buff=NEW_WCHAR(AllocSize);
        memcpy(buff,Text,sizeof(wchar_t)*pos);
      }
      else
      {
        buff=Text;
      }
      new_p=buff+TextLength+(source_len-replaced_len)-1;
      old_p=buff+TextLength-1;
      end_p=buff+pos+replaced_len;
      while(old_p>=end_p)
        *new_p--=*old_p--;

      if(buff!=Text)
      {
        DEL_WCHAR(Text);
        Text=buff;
      }
      TextLength+=source_len-replaced_len;
    }
    Text[TextLength]=L'\0';
  }
  return *this;
}
LWideString &LWideString::Replace(int pos,int replaced_len,LWideString &source)
{
  if(TextLength==0)
  {
    return operator=(source);
  }
  else
  if(TextLength<=pos)
  {
    return Append(source);
  }
  else
  if(TextLength<=pos+replaced_len)
  {
    TextLength=pos;
    return Append(source);
  }
  else
  {
    int source_len=source.GetLength();
    wchar_t *new_p,*old_p,*end_p;

    if(replaced_len>=source_len)
    {
      new_p=Text+pos+source_len;
      old_p=Text+pos+replaced_len;
      end_p=Text+TextLength;
      while(old_p<end_p)
        *new_p++=*old_p++;

      memcpy(Text+pos,source.GetText(),sizeof(wchar_t)*source_len);
      TextLength-=replaced_len-source_len;
    }
    else
    {
      wchar_t *buff;
      if(AllocSize<TextLength+source_len)
      {
        AllocSize=((AllocSize+(source_len-replaced_len)+128)/128+1)*128;
        buff=NEW_WCHAR(AllocSize);
        memcpy(buff,Text,sizeof(wchar_t)*pos);
      }
      else
      {
        buff=Text;                  
      }
      new_p=buff+TextLength+(source_len-replaced_len)-1;
      old_p=Text+TextLength-1;
      end_p=Text+pos+replaced_len;
      while(old_p>=end_p)
        *new_p--=*old_p--;

      memcpy(buff+pos,source.GetText(),sizeof(wchar_t)*source_len);

      if(buff!=Text)
      {
        DEL_WCHAR(Text);
        Text=buff;
      }
      TextLength+=source_len-replaced_len;
    }
    Text[TextLength]=L'\0';
  }
  return *this;
}
int LWideString::Compare(wchar_t* large_then_me)const
{
  if(Text==NULL && large_then_me==NULL)
    return 0;
  if(Text==NULL && large_then_me!=NULL)
    return 1;
  if(Text!=NULL && large_then_me==NULL)
    return -1;

  int pos;
  for(pos=0;large_then_me[pos] && pos<TextLength;pos++)
  {
    if(Text[pos]>large_then_me[pos])
      return -1;
    if(Text[pos]<large_then_me[pos])
      return 1;
  }
  if(Text[pos]>large_then_me[pos])
    return -1;
  if(Text[pos]<large_then_me[pos])
    return 1;
  return 0;
}
int LWideString::Compare(unsigned char *large_then_me)const
{
  if(Text==NULL && large_then_me==NULL)
    return 0;
  if(Text==NULL && large_then_me!=NULL)
    return 1;
  if(Text!=NULL && large_then_me==NULL)
    return -1;

  int ret;
  int len=strlen((char *)large_then_me);
  wchar_t *t=new wchar_t[len+1];

  MultiBytesToWides(t,(char *)large_then_me,len);
  ret=Compare(t);

  delete[] t;

  return ret;
}
int LWideString::Compare(wchar_t *large_then_me,int char_len)const
{
  if(Text==NULL && large_then_me==NULL)
    return 0;
  if(Text==NULL && large_then_me!=NULL)
    return 1;
  if(Text!=NULL && large_then_me==NULL)
    return -1;

  int pos;
  for(pos=0;pos<char_len && pos<TextLength;pos++)
  {
    if(Text[pos]>large_then_me[pos])
      return -1;
    if(Text[pos]<large_then_me[pos])
      return 1;
  }
  if(Text[pos]>large_then_me[pos])
    return -1;
  if(Text[pos]<large_then_me[pos])
    return 1;
  return 0;
}
int LWideString::Compare(unsigned char *large_then_me,int char_len)const
{
  if(Text==NULL && large_then_me==NULL)
    return 0;
  if(Text==NULL && large_then_me!=NULL)
    return 1;
  if(Text!=NULL && large_then_me==NULL)
    return -1;

  int ret;
  wchar_t *t=new wchar_t[char_len+1];
  ret=Compare(t);
  delete[] t;

  return ret;
}
int LWideString::Compare(const LWideString &s)const
{
  return Compare(&s);
}
int LWideString::Compare(const LWideString *s)const
{
  return Compare(s->GetText(),s->GetLength());
}
void LWideString::Trim()
{
  int index,start,end;

  if(TextLength==0)return;

  for(start=0;Text[start];start++)
    if(Text[start]!=L' ')
      break;
  for(end=TextLength-1;end>start;end--)
    if(Text[end]!=L' ')
      break;

  if(start>0)
  {
    for(index=0;start<=end;start++,index++)
      Text[index]=Text[start];
  }else
  {
    index=end+1-start;
  }
  Text[index]=0;
  TextLength=index;
}
size_t LWideString::Str(wchar_t *str)
{
  if(Text==NULL || str==NULL)
    return 0;

  wchar_t *t=wcsstr(Text,str);
  if(t==NULL)
    return 0;

  return 1+(((size_t)t)-((size_t)Text))/sizeof(wchar_t);
}
LWideString LWideString::SubString(int pos,int len)
{
  LWideString ret;
  size_t rational_len;

  if(pos>TextLength)
    return ret;

  if(pos+len<=TextLength+1)
    rational_len=len;
  else
    rational_len=TextLength+1-pos;

  ret.Copy(Text+pos-1,rational_len);

  return ret;
}
//---------------------------------------------------------------------------
#include <windows.h>

#ifdef __cplusplus
extern "C"{
#endif

size_t MultiBytesToWides(wchar_t *pwcs, const char *s, size_t n)
{
  size_t len=MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,s,n,pwcs,n);
  pwcs[len]=L'\0';
  return len;
}
size_t WidesToMultiBytes(char *s,const wchar_t *pwcs, size_t n)
{
  size_t len=WideCharToMultiByte(CP_ACP,WC_COMPOSITECHECK,pwcs,n,s,n*2,NULL,FALSE);
  s[len]='\0';
  return len;
}
void ToLower(wchar_t *pwcs)
{
  if(pwcs)
  {
    while(*pwcs)
    {
      *pwcs=towlower(*pwcs);
      pwcs++;
    }
  }
}
#ifdef __cplusplus
}
#endif

//---------------------------------------------------------------------------
LWideStrings::LWideStrings():LineCount(0),AllocSize(0),Strings(NULL)
{
}
LWideStrings::~LWideStrings()
{
  if(Strings!=NULL)
  {
    for(int i=0;i<LineCount;i++)
    {
      delete Strings[i];
    }
    delete[] Strings;
    Strings=NULL;
  }
}
int LWideStrings::GetLineCount()
{
  return LineCount;
}
void LWideStrings::Clear()
{
  if(Strings!=NULL)
  {
    for(int i=0;i<LineCount;i++)
    {
      Strings[i]->Clear();
    }
  }
}
LWideString *LWideStrings::GetLine(int index)
{
  if(index<LineCount)
    return Strings[index];
  return NULL;
}
void LWideStrings::GrowSize()
{
  if(Strings==NULL)
  {
    Strings=new LWideString*[1024];
    memset(Strings,0,1024*sizeof(LWideString*));
    AllocSize=1024;
    return;
  }

  LWideString ** store=Strings;
  Strings=new LWideString *[LineCount+1024];
  //copy old data
  for(int i=0;i<LineCount;i++)
  {
    Strings[i]=store[i];
  }
  memset(Strings+LineCount,0,1024*sizeof(LWideString *));
  AllocSize=LineCount+1024;
}
void LWideStrings::Add(LWideString &s)
{
  Add(&s);
}
void LWideStrings::Add(LWideString *s)
{
  if(LineCount==AllocSize)
  {
    GrowSize();
  }
  Strings[LineCount]=new LWideString;
  *Strings[LineCount]=*s;
  LineCount++;
}
void LWideStrings::Copy(LWideStrings &s)
{
  int i;
  LWideString *ws;
  for(i=0;i<s.LineCount;i++)
  {
    ws=s.GetLine(i);
    Add(ws);
  }
}
bool LWideStrings::Contain(LWideString &s)
{
  return Contain(&s);
}
bool LWideStrings::Contain(LWideString *s)
{
  int i;
  LWideString *ws;
  for(i=0;i<LineCount;i++)
  {
    ws=GetLine(i);
    if(ws->Compare(s)==0)
      return true;
  }
  return false;
}
//---------------------------------------------------------------------------

