// Copyright (c) Luodiping 2008
// http://duceland.com

#ifndef __DTYPES_H__
#define __DTYPES_H__
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef __BORLANDC__
  #include <vcl.h>
#endif

#if defined(__GNUC__)
  #define _wtof(a) wcstof(a,NULL)
  #define _wtold(a) wcstod(a,NULL)
#endif
#ifdef _MSC_VER
  #define _wtof(a) wcstod(a,NULL)
  #define _wtold(a) wcstod(a,NULL)

  #include <wtypes.h>
#endif

#include "LWideString.h"

#pragma pack(4)

//[published declear- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#ifndef DNI_TYPES_H
typedef unsigned int     dbool;
typedef unsigned char    dbyte;
typedef char*            dascii;
typedef wchar_t          dchar;
typedef short            dshort;
typedef float            dfloat;
typedef double           ddouble;
typedef long double      dld;
typedef int              dint;
typedef LWideString*     dstring;
typedef unsigned __int64 dsize;
typedef dchar*           dchararray;

struct _dobject;
typedef _dobject*        dobject;
#endif

#define vt_unknown    0x00000000
#define vt_bool       0x10000001
#define vt_byte       0x10000002
#define vt_ascii      0x10000003
#define vt_char       0x10000004
#define vt_short      0x10000005
#define vt_float      0x10000006
#define vt_double     0x10000007
#define vt_ld         0x10000008
#define vt_int        0x10000009
#define vt_size       0x10000010
#define vt_string     0x10000012
#define vt_void       0x10000013
#define vt_object     0x10000200
#define vt_lobject    0x10000201
#define vt_vclobject  0x20000000
#define vt_mfcobject  0x30000000
#define vt_custom     0x40000000//for example, TPoint is vt_custom+sizeof(TPoint),CRect is vt_custom+sizeof(CRect)

struct dValue
{
  dint vt;//variant teype
  union{
    dbool     b;
    dbyte     y;
    dchar     c;
    dascii    a;
    dshort    s;
    dfloat    f;
    ddouble   d;
    dld       l;
    dint      i;
    dsize     z;
    dbyte     reserved[32];//must be 32 bytes
  }v;
  struct{
    dstring   s;
    dobject   o;
  }o;
  dValue   *an;//array next

  //- - - - - - - -- - - - -- - -
  inline dValue()//default constructor
  {
    memset(this,0,sizeof(*this));
    vt=vt_unknown;
  }
  inline dValue(const dValue& rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dValue& rhs)//set value operator
  {
    if(rhs.vt==vt_string)
    {
      if(rhs.o.s)
        *this=rhs.o.s->GetText();
      else
        *this=(dchararray)NULL;
    }else
    {
      memcpy(this,&rhs,sizeof(dValue));
    }
    return *this;
  }

  // - support bool
  inline dValue(bool rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue(dbool rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const bool& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_bool;
    v.b=rhs;
    return *this;
  }
  inline dValue& operator =(const dbool& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_bool;
    v.b=rhs;
    return *this;
  }
  inline operator dbool()//get value operator
  {
    return AsBool();
  }
  inline dbool AsBool()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      if(v.f==0.0)
        return 0;
      else
        return 1;
    case vt_double:
      if(v.d==0.0f)
        return 0;
      else
        return 1;
    case vt_ld:
      if(v.l==0.0f)
        return 0;
      else
        return 1;
    case vt_int:
      return v.i;
    case vt_size:
      return (dbool)v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return 1;
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.i;
    }
  }

  // - support byte
  inline dValue(dbyte rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dbyte& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_byte;
    v.y=rhs;
    return *this;
  }
  inline operator dbyte()//get value operator
  {
    return AsByte();
  }
  inline dbyte AsByte()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return (dbyte)v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return (dbyte)v.c;
    case vt_short:
      return (dbyte)v.s;
    case vt_float:
      return (dbyte)v.f;
    case vt_double:
      return (dbyte)v.d;
    case vt_ld:
      return (dbyte)v.l;
    case vt_int:
      return (dbyte)v.i;
    case vt_size:
      return (dbyte)v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return (dbyte)_wtoi(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.b;
    }
  }

  // - support wchar_t-dchar
  inline dValue(dchar rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dchar& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_char;
    v.c=rhs;
    return *this;
  }
  inline operator dchar()//get value operator
  {
    return AsChar();
  }
  inline dchar AsChar()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return (dchar)v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return (dchar)v.f;
    case vt_double:
      return (dchar)v.d;
    case vt_ld:
      return (dchar)v.l;
    case vt_int:
      return (dchar)v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return (dchar)_wtoi(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.s;
    }
  }

  // - support short
  inline dValue(dshort rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dshort& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_short;
    v.s=rhs;
    return *this;
  }
  inline operator dshort()//get value operator
  {
    return AsShort();
  }
  inline dshort AsShort()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return v.l;
    case vt_int:
      return v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtoi(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.s;
    }
  }

  // - support float
  inline dValue(dfloat rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dfloat& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_float;
    v.f=rhs;
    return *this;
  }
  inline operator dfloat()//get value operator
  {
    return AsFloat();
  }
  inline dfloat AsFloat()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return v.l;
    case vt_int:
      return v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtof(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0.0f;
      else
        return 1.0f;
    default:
      return v.d;
    }
  }

  // - support double
  inline dValue(ddouble rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const ddouble& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_double;
    v.d=rhs;
    return *this;
  }
  inline operator ddouble()//get value operator
  {
    return AsDouble();
  }
  inline ddouble AsDouble()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return v.l;
    case vt_int:
      return v.i;
    case vt_size:
      #ifdef _MSC_VER
      {
        char a[20];
        _i64toa(v.z,a,10);
        return atof(a);
      }
      #else
        return v.z;
      #endif
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtold(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0.0;
      else
        return 1.0;
    default:
      return v.d;
    }
  }

  // - support long double
  inline dValue(dld rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dld& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_ld;
    v.l=rhs;
    return *this;
  }
  inline operator dld()//get value operator
  {
    return AsLongDouble();
  }
  inline dld AsLongDouble()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return v.l;
    case vt_int:
      return v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtold(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0.0;
      else
        return 1.0;
    default:
      return v.l;
    }
  }

  // - support int
  inline dValue(dint rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dint& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_int;
    v.i=rhs;
    return *this;
  }
  inline operator dint()//get value operator
  {
    return AsInt();
  }
  inline dint AsInt()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return v.l;
    case vt_int:
      return v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtoi(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.i;
    }
  }

  // - support size
  inline dValue(dsize rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dsize& rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_size;
    v.z=rhs;
    return *this;
  }
  inline operator dsize()//get value operator
  {
    return AsSize();
  }
  inline dsize AsSize()//get value method
  {
    switch(vt)
    {
    case vt_bool:
      return v.b;
    case vt_byte:
      return v.y;
    case vt_char:
      return v.c;
    case vt_short:
      return v.s;
    case vt_float:
      return v.f;
    case vt_double:
      return v.d;
    case vt_ld:
      return (dsize)v.l;
    case vt_int:
      return v.i;
    case vt_size:
      return v.z;
    case vt_string:
      if(o.s==NULL)
        return 0;
      if(o.s->GetText()==NULL)
        return 0;
      if(o.s->GetText()[0]==L'0')
        return 0;
      return _wtoi64(o.s->GetText());
    case vt_object:
      if(o.o==NULL)
        return 0;
      else
        return 1;
    default:
      return v.z;
    }
  }

  // - support wchar_t * - dchararray
  inline dValue(dchararray rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dchararray rhs);//set value operator. implement in dni.hpp
  inline operator dstring()//get value operator
  {
    return AsString();
  }
  inline operator dchararray()//get value operator
  {
    return AsString()->GetText();
  }
  inline dstring AsString()//get value method
  {
    if(vt==vt_string)
      return o.s;
    dchar s[60];
    if(vt==vt_float || vt==vt_double || vt==vt_ld)
    {
      double d=AsDouble();
      swprintf(s,L"%f",d);
    }else if(vt==vt_bool || vt==vt_byte || vt==vt_char || vt==vt_int || vt==vt_size)
    {
      dint i=AsInt();
      swprintf(s,L"%d",i);
    }else if(vt>vt_custom && vt<=(vt_custom+0x1000))
    {
      int sz=vt-vt_custom;
      s[0]=L'(';
      int pos=1;
      for(int i=0;i<sz;i++)
      {
        if(i+1<sz)
          pos+=swprintf(s+pos,L"%d,",v.reserved[i]);
        else
          pos+=swprintf(s+pos,L"%d",v.reserved[i]);
      }
      s[pos++]=L')';
      s[pos]=0;
    }else
    if(vt==vt_ascii)
    {
      if(o.s==NULL)
        Ascii2String();
      return o.s;
    }else
    {
      s[0]=0;
    }
    dValue s_temp=s;
    o.s=s_temp.o.s;
    s_temp.o.s=NULL;
    s_temp.vt=vt_unknown;
    return o.s;
  }

  // - support ASCII - char *
  inline dValue(dascii rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dascii rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_ascii;
    v.a=rhs;
    return *this;
  }
  inline operator dascii()//get value operator
  {
    return AsAscii();
  }
  inline dascii AsAscii()//get value method
  {
    if(vt==vt_ascii)
    {
      return v.a;
    }
    return NULL;
  }

  // - support dobject
  inline dValue(dobject rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const dobject rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_object;
    o.o=rhs;
    return *this;
  }
  inline operator dobject()//get value operator
  {
    return AsObject();
  }
  inline dobject AsObject()//get value method
  {
    if(vt==vt_object || vt==vt_lobject)
      return o.o;
    return NULL;
  }

  // - support GUID
  inline dValue(GUID rhs)//constructor
  {
    operator =(rhs);
  }
  inline dValue& operator =(const GUID &rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_lobject;
    memcpy(v.reserved,&rhs,sizeof(rhs));
    return *this;
  }
  inline operator GUID()//get value operator
  {
    return AsGUID();
  }
  inline GUID AsGUID()//get value method
  {
    return *((GUID *)v.reserved);
  }

  // - support void *
  inline dValue(const void *rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const void * rhs)//set value operator
  {
    CheckObject(false);
    vt=vt_lobject;
    o.o=(dobject)rhs;
    return *this;
  }
  inline operator void *()//get value operator
  {
    return AsNativeObject();
  }
  inline void * AsNativeObject();//get value method. implement in dni.hpp

  #ifdef __BORLANDC__
  // - support TPoint
  inline dValue(TPoint rhs)//constructor
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const TPoint& rhs)
  {
    CheckObject(false);
    POINT p;
    vt=vt_vclobject+*(int *)"Poi";
    p.x=rhs.x;p.y=rhs.y;
    memcpy(v.reserved,&p,sizeof(POINT));
    return *this;
  }
  // - support TRect
  inline dValue(TRect rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const TRect& rhs)
  {
    CheckObject(false);
    RECT r;
    vt=vt_vclobject+*(int *)"Rec";
    r.left=rhs.left;r.top=rhs.top;r.right=rhs.right;r.bottom=rhs.bottom;
    memcpy(v.reserved,&r,sizeof(RECT));
    return *this;
  }
  // - support TWMKey
  inline dValue(TWMKey rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const TWMKey& rhs)
  {
    CheckObject(false);
    vt=vt_vclobject+*(int *)"WMK";
    memcpy(v.reserved,&rhs,sizeof(TWMKey));
    return *this;
  }
  // - support AnsiString
  inline dValue(AnsiString rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const AnsiString& rhs)
  {
    operator=(rhs.c_str());
    return operator=(AsString()->GetText());
  }
  // - support enum TDragState ( to int)
  inline dValue(TDragState rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const TDragState& rhs)
  {
    CheckObject(false);
    vt=vt_int;
    v.i=rhs;
    return *this;
  }
  // - support enum TShiftState ( to int)
  inline dValue(TShiftState rhs)
  {
    memset(this,0,sizeof(*this));
    operator =(rhs);
  }
  inline dValue& operator =(const TShiftState& rhs)
  {
    CheckObject(false);
    vt=vt_int;
    v.i=0;
    if(rhs.Contains(ssShift))
      v.i&=0x1;
    if(rhs.Contains(ssAlt))
      v.i&=0x2;
    if(rhs.Contains(ssCtrl))
      v.i&=0x4;
    if(rhs.Contains(ssLeft))
      v.i&=0x8;
    if(rhs.Contains(ssRight))
      v.i&=0x16;
    if(rhs.Contains(ssMiddle))
      v.i&=0x32;
    if(rhs.Contains(ssDouble))
      v.i&=0x64;
    return *this;
  }

  //convert to AnsiString
  inline operator AnsiString()
  {
    if(vt==vt_ascii)
      return v.a;
    AnsiString ret;
    ret=AsString()->GetText();
    return ret;
  }
  #endif//__BORLANDC__

  inline ~dValue();
  //- - - - - - - -- - - - -- - -

  // - support +-*/
  inline dValue operator +(dValue& rhs)
  {
    dValue ret;
    LWideString *s;
    switch(vt)
    {
    case vt_bool:// bool + ...
      ret.vt=vt_bool;
      ret.v.b=v.b || rhs.AsBool();
      return ret;
    case vt_byte:
      switch(rhs.vt)
      {
      case vt_bool://byte + bool=byte
        ret.vt=vt_byte;
        ret.v.y=v.y+rhs.AsByte();;
        return ret;
      case vt_byte://byte + byte=byte
        ret.vt=vt_byte;
        ret.v.y=v.y+rhs.v.y;
        return ret;
      case vt_char://byte + char=byte
        ret.vt=vt_byte;
        ret.v.y=v.y+rhs.v.c;
        return ret;
      case vt_short://byte + short=short
        ret.vt=vt_short;
        ret.v.s=v.y+rhs.v.s;
        return ret;
      case vt_float://byte + float=float
        ret.vt=vt_float;
        ret.v.f=v.y+rhs.v.f;
        return ret;
      case vt_double://byte + double=double
        ret.vt=vt_double;
        ret.v.d=v.y+rhs.v.d;
        return ret;
      case vt_ld://byte + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.y+rhs.v.l;
        return ret;
      case vt_int://byte + int=int
        ret.vt=vt_int;
        ret.v.i=v.y+rhs.v.i;
        return ret;
      case vt_size://byte + size=size
        ret.vt=vt_size;
        ret.v.z=v.y+rhs.v.z;
        return ret;
      case vt_string://byte + string = byte
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.y+rhs.AsByte();
        return ret;
      }
      return ret;
    case vt_char:
      switch(rhs.vt)
      {
      case vt_bool://char + bool=char
        ret.vt=vt_char;
        ret.v.c=v.c+rhs.v.b;
        return ret;
      case vt_byte://char + byte=char
        ret.vt=vt_char;
        ret.v.c=v.c+rhs.v.y;
        return ret;
      case vt_char://char + char=char
        ret.vt=vt_byte;
        ret.v.c=v.c+rhs.v.c;
        return ret;
      case vt_short://char + short=short
        ret.vt=vt_short;
        ret.v.s=v.c+rhs.v.s;
        return ret;
      case vt_float://char + float=float
        ret.vt=vt_float;
        ret.v.f=v.c+rhs.v.f;
        return ret;
      case vt_double://char + double=double
        ret.vt=vt_double;
        ret.v.d=v.c+rhs.v.d;
        return ret;
      case vt_ld://char + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.c+rhs.v.l;
        return ret;
      case vt_int://char + int=int
        ret.vt=vt_int;
        ret.v.i=v.c+rhs.v.i;
        return ret;
      case vt_size://char + size=size
        ret.vt=vt_size;
        ret.v.z=v.c+rhs.v.z;
        return ret;
      case vt_string://char + string = byte
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.c+rhs.AsByte();
        return ret;
      }
      return ret;
    case vt_short:
      switch(rhs.vt)
      {
      case vt_bool://short + bool=short
        ret.vt=vt_short;
        ret.v.s=v.s+rhs.v.b;
        return ret;
      case vt_byte://short + byte=short
        ret.vt=vt_short;
        ret.v.s=v.s+rhs.v.y;
        return ret;
      case vt_char://short + char=short
        ret.vt=vt_short;
        ret.v.s=v.s+rhs.v.c;
        return ret;
      case vt_short://short + short=short
        ret.vt=vt_short;
        ret.v.s=v.s+rhs.v.s;
        return ret;
      case vt_float://short + float=float
        ret.vt=vt_float;
        ret.v.f=v.s+rhs.v.f;
        return ret;
      case vt_double://short + double=double
        ret.vt=vt_double;
        ret.v.d=v.s+rhs.v.d;
        return ret;
      case vt_ld://short + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.s+rhs.v.l;
        return ret;
      case vt_int://short + int=int
        ret.vt=vt_int;
        ret.v.i=v.s+rhs.v.i;
        return ret;
      case vt_size://short + size=size
        ret.vt=vt_size;
        ret.v.z=v.s+rhs.v.z;
        return ret;
      case vt_string://short + string=byte
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.s+rhs.AsShort();
        return ret;
      }
      return ret;
    case vt_float:
      switch(rhs.vt)
      {
      case vt_bool://float + bool=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.b;
        return ret;
      case vt_byte://float + byte=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.y;
        return ret;
      case vt_char://float + char=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.c;
        return ret;
      case vt_short://float + short=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.s;
        return ret;
      case vt_float://float + float=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.f;
        return ret;
      case vt_double://float + double=double
        ret.vt=vt_double;
        ret.v.d=v.f+rhs.v.d;
        return ret;
      case vt_ld://float + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.f+rhs.v.l;
        return ret;
      case vt_int://float + int=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.i;
        return ret;
      case vt_size://float + size=float
        ret.vt=vt_float;
        ret.v.f=v.f+rhs.v.z;
        return ret;
      case vt_string://float + string=float
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.s+rhs.AsFloat();
        return ret;
      }
      return ret;
    case vt_double:
      switch(rhs.vt)
      {
      case vt_bool://double + bool=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.b;
        return ret;
      case vt_byte://double + byte=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.y;
        return ret;
      case vt_char://double + char=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.c;
        return ret;
      case vt_short://double + short=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.s;
        return ret;
      case vt_float://double + float=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.f;
        return ret;
      case vt_double://double + double=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.d;
        return ret;
      case vt_int://double + int=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.i;
        return ret;
      case vt_size://double + size=double
        ret.vt=vt_double;
        ret.v.d=v.d+rhs.v.z;
        return ret;
      case vt_string://double + string=double
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.s+rhs.AsDouble();
        return ret;
      }
      return ret;
    case vt_int:
      switch(rhs.vt)
      {
      case vt_bool://int + bool=int
        ret.vt=vt_int;
        ret.v.i=v.i+rhs.v.b;
        return ret;
      case vt_byte://int + byte=int
        ret.vt=vt_int;
        ret.v.i=v.i+rhs.v.y;
        return ret;
      case vt_char://int + char=int
        ret.vt=vt_int;
        ret.v.i=v.i+rhs.v.c;
        return ret;
      case vt_short://int + short=int
        ret.vt=vt_int;
        ret.v.i=v.i+rhs.v.s;
        return ret;
      case vt_float://int + float=float
        ret.vt=vt_float;
        ret.v.f=v.i+rhs.v.f;
        return ret;
      case vt_double://int + double=double
        ret.vt=vt_double;
        ret.v.d=v.i+rhs.v.d;
        return ret;
      case vt_ld://int + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.i+rhs.v.l;
        return ret;
      case vt_int://int + int=int
        ret.vt=vt_int;
        ret.v.i=v.i+rhs.v.i;
        return ret;
      case vt_size://int + size=size
        ret.vt=vt_size;
        ret.v.z=v.i+rhs.v.z;
        return ret;
      case vt_string://int + string=int
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.s+rhs.AsInt();
        return ret;
      }
      return ret;
    case vt_size:
      switch(rhs.vt)
      {
      case vt_bool://size + bool=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.b;
        return ret;
      case vt_byte://size + byte=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.y;
        return ret;
      case vt_char://size + char=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.c;
        return ret;
      case vt_short://size + short=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.s;
        return ret;
      case vt_float://size + float=float
        ret.vt=vt_float;
        ret.v.f=v.z+rhs.v.f;
        return ret;
      case vt_double://size + double=double
        ret.vt=vt_double;
        ret.v.d=v.z+rhs.v.d;
        return ret;
      case vt_ld://size + long double=long double
        ret.vt=vt_ld;
        ret.v.l=v.z+rhs.v.l;
        return ret;
      case vt_int://size + int=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.i;
        return ret;
      case vt_size://size + size=size
        ret.vt=vt_size;
        ret.v.z=v.z+rhs.v.z;
        return ret;
      case vt_string://size + string=size
      case vt_ascii:
        ret.vt=vt_short;
        ret.v.z=v.s+rhs.AsSize();
        return ret;
      }
      return ret;
    case vt_string://string + xx
      ret=*this;
      (*ret.o.s)+=rhs.AsString()->GetText();
      return ret;
    case vt_ascii:
      if(rhs.vt==vt_ascii || rhs.vt==vt_string)
      {
        s=AsString();
        if(s==NULL || s->GetLength()==0)
        {
          ret=rhs;
          return ret;
        }

        ret=*this;
        (*ret.o.s)+=rhs.AsString()->GetText();
      }else
      {
        ret=v.a[rhs.AsInt()];
      }
      return ret;
    }
    return ret;
  }


  // - support += ,-=, *=, /=
  inline dValue& operator +=(dValue& rhs)
  {
		switch(vt)
    {
    case vt_bool:// bool + ...
      vt=vt_bool;
      v.b+=v.b || rhs.AsBool();
      return *this;
    case vt_byte:
      switch(rhs.vt)
      {
      case vt_bool://byte + bool=byte
        vt=vt_byte;
        v.y+=v.y+rhs.AsByte();;
        return *this;
      case vt_byte://byte + byte=byte
        vt=vt_byte;
        v.y+=v.y+rhs.v.y;
        return *this;
      case vt_char://byte + char=byte
        vt=vt_byte;
        v.y+=v.y+rhs.v.c;
        return *this;
      case vt_short://byte + short=short
        vt=vt_short;
        v.s+=v.y+rhs.v.s;
        return *this;
      case vt_float://byte + float=float
        vt=vt_float;
        v.f+=v.y+rhs.v.f;
        return *this;
      case vt_double://byte + double=double
        vt=vt_double;
        v.d+=v.y+rhs.v.d;
        return *this;
      case vt_ld://byte + long double=long double
        vt=vt_ld;
        v.l+=v.y+rhs.v.l;
        return *this;
      case vt_int://byte + int=int
        vt=vt_int;
        v.i+=v.y+rhs.v.i;
        return *this;
      case vt_size://byte + size=size
        vt=vt_size;
        v.z+=v.y+rhs.v.z;
        return *this;
      case vt_string://byte + string = byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.y+rhs.AsByte();
        return *this;
      }
      return *this;
    case vt_char:
      switch(rhs.vt)
      {
      case vt_bool://char + bool=char
        vt=vt_char;
        v.c+=v.c+rhs.v.b;
        return *this;
      case vt_byte://char + byte=char
        vt=vt_char;
        v.c+=v.c+rhs.v.y;
        return *this;
      case vt_char://char + char=char
        vt=vt_byte;
        v.c+=v.c+rhs.v.c;
        return *this;
      case vt_short://char + short=short
        vt=vt_short;
        v.s+=v.c+rhs.v.s;
        return *this;
      case vt_float://char + float=float
        vt=vt_float;
        v.f+=v.c+rhs.v.f;
        return *this;
      case vt_double://char + double=double
        vt=vt_double;
        v.d+=v.c+rhs.v.d;
        return *this;
      case vt_ld://char + long double=long double
        vt=vt_ld;
        v.l+=v.c+rhs.v.l;
        return *this;
      case vt_int://char + int=int
        vt=vt_int;
        v.i+=v.c+rhs.v.i;
        return *this;
      case vt_size://char + size=size
        vt=vt_size;
        v.z+=v.c+rhs.v.z;
        return *this;
      case vt_string://char + string = byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.c+rhs.AsByte();
        return *this;
      }
      return *this;
    case vt_short:
      switch(rhs.vt)
      {
      case vt_bool://short + bool=short
        vt=vt_short;
        v.s+=v.s+rhs.v.b;
        return *this;
      case vt_byte://short + byte=short
        vt=vt_short;
        v.s+=v.s+rhs.v.y;
        return *this;
      case vt_char://short + char=short
        vt=vt_short;
        v.s+=v.s+rhs.v.c;
        return *this;
      case vt_short://short + short=short
        vt=vt_short;
        v.s+=v.s+rhs.v.s;
        return *this;
      case vt_float://short + float=float
        vt=vt_float;
        v.f+=v.s+rhs.v.f;
        return *this;
      case vt_double://short + double=double
        vt=vt_double;
        v.d+=v.s+rhs.v.d;
        return *this;
      case vt_ld://short + long double=long double
        vt=vt_ld;
        v.l+=v.s+rhs.v.l;
        return *this;
      case vt_int://short + int=int
        vt=vt_int;
        v.i+=v.s+rhs.v.i;
        return *this;
      case vt_size://short + size=size
        vt=vt_size;
        v.z+=v.s+rhs.v.z;
        return *this;
      case vt_string://short + string=byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs.AsShort();
        return *this;
      }
      return *this;
    case vt_float:
      switch(rhs.vt)
      {
      case vt_bool://float + bool=float
        vt=vt_float;
        v.f+=v.f+rhs.v.b;
        return *this;
      case vt_byte://float + byte=float
        vt=vt_float;
        v.f+=v.f+rhs.v.y;
        return *this;
      case vt_char://float + char=float
        vt=vt_float;
        v.f+=v.f+rhs.v.c;
        return *this;
      case vt_short://float + short=float
        vt=vt_float;
        v.f+=v.f+rhs.v.s;
        return *this;
      case vt_float://float + float=float
        vt=vt_float;
        v.f+=v.f+rhs.v.f;
        return *this;
      case vt_double://float + double=double
        vt=vt_double;
        v.d+=v.f+rhs.v.d;
        return *this;
      case vt_ld://float + long double=long double
        vt=vt_ld;
        v.l+=v.f+rhs.v.l;
        return *this;
      case vt_int://float + int=float
        vt=vt_float;
        v.f+=v.f+rhs.v.i;
        return *this;
      case vt_size://float + size=float
        vt=vt_float;
        v.f+=v.f+rhs.v.z;
        return *this;
      case vt_string://float + string=float
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs.AsFloat();
        return *this;
      }
      return *this;
    case vt_double:
      switch(rhs.vt)
      {
      case vt_bool://double + bool=double
        vt=vt_double;
        v.d+=v.d+rhs.v.b;
        return *this;
      case vt_byte://double + byte=double
        vt=vt_double;
        v.d+=v.d+rhs.v.y;
        return *this;
      case vt_char://double + char=double
        vt=vt_double;
        v.d+=v.d+rhs.v.c;
        return *this;
      case vt_short://double + short=double
        vt=vt_double;
        v.d+=v.d+rhs.v.s;
        return *this;
      case vt_float://double + float=double
        vt=vt_double;
        v.d+=v.d+rhs.v.f;
        return *this;
      case vt_double://double + double=double
        vt=vt_double;
        v.d+=v.d+rhs.v.d;
        return *this;
      case vt_int://double + int=double
        vt=vt_double;
        v.d+=v.d+rhs.v.i;
        return *this;
      case vt_size://double + size=double
        vt=vt_double;
        v.d+=v.d+rhs.v.z;
        return *this;
      case vt_string://double + string=double
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs.AsDouble();
        return *this;
      }
      return *this;
    case vt_int:
      switch(rhs.vt)
      {
      case vt_bool://int + bool=int
        vt=vt_int;
        v.i+=v.i+rhs.v.b;
        return *this;
      case vt_byte://int + byte=int
        vt=vt_int;
        v.i+=v.i+rhs.v.y;
        return *this;
      case vt_char://int + char=int
        vt=vt_int;
        v.i+=v.i+rhs.v.c;
        return *this;
      case vt_short://int + short=int
        vt=vt_int;
        v.i+=v.i+rhs.v.s;
        return *this;
      case vt_float://int + float=float
        vt=vt_float;
        v.f+=v.i+rhs.v.f;
        return *this;
      case vt_double://int + double=double
        vt=vt_double;
        v.d+=v.i+rhs.v.d;
        return *this;
      case vt_ld://int + long double=long double
        vt=vt_ld;
        v.l+=v.i+rhs.v.l;
        return *this;
      case vt_int://int + int=int
        vt=vt_int;
        v.i+=v.i+rhs.v.i;
        return *this;
      case vt_size://int + size=size
        vt=vt_size;
        v.z+=v.i+rhs.v.z;
        return *this;
      case vt_string://int + string=int
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs.AsInt();
        return *this;
      }
      return *this;
    case vt_size:
      switch(rhs.vt)
      {
      case vt_bool://size + bool=size
        vt=vt_size;
        v.z+=v.z+rhs.v.b;
        return *this;
      case vt_byte://size + byte=size
        vt=vt_size;
        v.z+=v.z+rhs.v.y;
        return *this;
      case vt_char://size + char=size
        vt=vt_size;
        v.z+=v.z+rhs.v.c;
        return *this;
      case vt_short://size + short=size
        vt=vt_size;
        v.z+=v.z+rhs.v.s;
        return *this;
      case vt_float://size + float=float
        vt=vt_float;
        v.f+=v.z+rhs.v.f;
        return *this;
      case vt_double://size + double=double
        vt=vt_double;
        v.d+=v.z+rhs.v.d;
        return *this;
      case vt_ld://size + long double=long double
        vt=vt_ld;
        v.l+=v.z+rhs.v.l;
        return *this;
      case vt_int://size + int=size
        vt=vt_size;
        v.z+=v.z+rhs.v.i;
        return *this;
      case vt_size://size + size=size
        vt=vt_size;
        v.z+=v.z+rhs.v.z;
        return *this;
      case vt_string://size + string=size
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs.AsSize();
        return *this;
      }
      return *this;
    case vt_string:
      (*o.s)+=*rhs.AsString();
      return *this;
    case vt_ascii:
      if(rhs.vt==vt_ascii || rhs.vt==vt_string)
      {
        AsString();
        (*o.s)+=rhs.AsString()->GetText();
      }
      return *this;
    default:
      operator =(rhs);
    }
    return *this;
  }
	inline dValue& operator +=(dchar rhs)
	{
		dValue rhs_v=rhs;
		switch(vt)
    {
    case vt_bool:// bool + ...
      vt=vt_bool;
      v.b+=v.b || rhs_v.AsBool();
      return *this;
    case vt_byte:
      switch(rhs_v.vt)
      {
      case vt_bool://byte + bool=byte
        vt=vt_byte;
        v.y+=v.y+rhs_v.AsByte();;
        return *this;
      case vt_byte://byte + byte=byte
        vt=vt_byte;
        v.y+=v.y+rhs_v.v.y;
        return *this;
      case vt_char://byte + char=byte
        vt=vt_byte;
        v.y+=v.y+rhs_v.v.c;
        return *this;
      case vt_short://byte + short=short
        vt=vt_short;
        v.s+=v.y+rhs_v.v.s;
        return *this;
      case vt_float://byte + float=float
        vt=vt_float;
        v.f+=v.y+rhs_v.v.f;
        return *this;
      case vt_double://byte + double=double
        vt=vt_double;
        v.d+=v.y+rhs_v.v.d;
        return *this;
      case vt_ld://byte + long double=long double
        vt=vt_ld;
        v.l+=v.y+rhs_v.v.l;
        return *this;
      case vt_int://byte + int=int
        vt=vt_int;
        v.i+=v.y+rhs_v.v.i;
        return *this;
      case vt_size://byte + size=size
        vt=vt_size;
        v.z+=v.y+rhs_v.v.z;
        return *this;
      case vt_string://byte + string = byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.y+rhs_v.AsByte();
        return *this;
      }
      return *this;
    case vt_char:
      switch(rhs_v.vt)
      {
      case vt_bool://char + bool=char
        vt=vt_char;
        v.c+=v.c+rhs_v.v.b;
        return *this;
      case vt_byte://char + byte=char
        vt=vt_char;
        v.c+=v.c+rhs_v.v.y;
        return *this;
      case vt_char://char + char=char
        vt=vt_byte;
        v.c+=v.c+rhs_v.v.c;
        return *this;
      case vt_short://char + short=short
        vt=vt_short;
        v.s+=v.c+rhs_v.v.s;
        return *this;
      case vt_float://char + float=float
        vt=vt_float;
        v.f+=v.c+rhs_v.v.f;
        return *this;
      case vt_double://char + double=double
        vt=vt_double;
        v.d+=v.c+rhs_v.v.d;
        return *this;
      case vt_ld://char + long double=long double
        vt=vt_ld;
        v.l+=v.c+rhs_v.v.l;
        return *this;
      case vt_int://char + int=int
        vt=vt_int;
        v.i+=v.c+rhs_v.v.i;
        return *this;
      case vt_size://char + size=size
        vt=vt_size;
        v.z+=v.c+rhs_v.v.z;
        return *this;
      case vt_string://char + string = byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.c+rhs_v.AsByte();
        return *this;
      }
      return *this;
    case vt_short:
      switch(rhs_v.vt)
      {
      case vt_bool://short + bool=short
        vt=vt_short;
        v.s+=v.s+rhs_v.v.b;
        return *this;
      case vt_byte://short + byte=short
        vt=vt_short;
        v.s+=v.s+rhs_v.v.y;
        return *this;
      case vt_char://short + char=short
        vt=vt_short;
        v.s+=v.s+rhs_v.v.c;
        return *this;
      case vt_short://short + short=short
        vt=vt_short;
        v.s+=v.s+rhs_v.v.s;
        return *this;
      case vt_float://short + float=float
        vt=vt_float;
        v.f+=v.s+rhs_v.v.f;
        return *this;
      case vt_double://short + double=double
        vt=vt_double;
        v.d+=v.s+rhs_v.v.d;
        return *this;
      case vt_ld://short + long double=long double
        vt=vt_ld;
        v.l+=v.s+rhs_v.v.l;
        return *this;
      case vt_int://short + int=int
        vt=vt_int;
        v.i+=v.s+rhs_v.v.i;
        return *this;
      case vt_size://short + size=size
        vt=vt_size;
        v.z+=v.s+rhs_v.v.z;
        return *this;
      case vt_string://short + string=byte
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs_v.AsShort();
        return *this;
      }
      return *this;
    case vt_float:
      switch(rhs_v.vt)
      {
      case vt_bool://float + bool=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.b;
        return *this;
      case vt_byte://float + byte=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.y;
        return *this;
      case vt_char://float + char=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.c;
        return *this;
      case vt_short://float + short=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.s;
        return *this;
      case vt_float://float + float=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.f;
        return *this;
      case vt_double://float + double=double
        vt=vt_double;
        v.d+=v.f+rhs_v.v.d;
        return *this;
      case vt_ld://float + long double=long double
        vt=vt_ld;
        v.l+=v.f+rhs_v.v.l;
        return *this;
      case vt_int://float + int=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.i;
        return *this;
      case vt_size://float + size=float
        vt=vt_float;
        v.f+=v.f+rhs_v.v.z;
        return *this;
      case vt_string://float + string=float
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs_v.AsFloat();
        return *this;
      }
      return *this;
    case vt_double:
      switch(rhs_v.vt)
      {
      case vt_bool://double + bool=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.b;
        return *this;
      case vt_byte://double + byte=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.y;
        return *this;
      case vt_char://double + char=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.c;
        return *this;
      case vt_short://double + short=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.s;
        return *this;
      case vt_float://double + float=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.f;
        return *this;
      case vt_double://double + double=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.d;
        return *this;
      case vt_int://double + int=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.i;
        return *this;
      case vt_size://double + size=double
        vt=vt_double;
        v.d+=v.d+rhs_v.v.z;
        return *this;
      case vt_string://double + string=double
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs_v.AsDouble();
        return *this;
      }
      return *this;
    case vt_int:
      switch(rhs_v.vt)
      {
      case vt_bool://int + bool=int
        vt=vt_int;
        v.i+=v.i+rhs_v.v.b;
        return *this;
      case vt_byte://int + byte=int
        vt=vt_int;
        v.i+=v.i+rhs_v.v.y;
        return *this;
      case vt_char://int + char=int
        vt=vt_int;
        v.i+=v.i+rhs_v.v.c;
        return *this;
      case vt_short://int + short=int
        vt=vt_int;
        v.i+=v.i+rhs_v.v.s;
        return *this;
      case vt_float://int + float=float
        vt=vt_float;
        v.f+=v.i+rhs_v.v.f;
        return *this;
      case vt_double://int + double=double
        vt=vt_double;
        v.d+=v.i+rhs_v.v.d;
        return *this;
      case vt_ld://int + long double=long double
        vt=vt_ld;
        v.l+=v.i+rhs_v.v.l;
        return *this;
      case vt_int://int + int=int
        vt=vt_int;
        v.i+=v.i+rhs_v.v.i;
        return *this;
      case vt_size://int + size=size
        vt=vt_size;
        v.z+=v.i+rhs_v.v.z;
        return *this;
      case vt_string://int + string=int
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs_v.AsInt();
        return *this;
      }
      return *this;
    case vt_size:
      switch(rhs_v.vt)
      {
      case vt_bool://size + bool=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.b;
        return *this;
      case vt_byte://size + byte=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.y;
        return *this;
      case vt_char://size + char=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.c;
        return *this;
      case vt_short://size + short=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.s;
        return *this;
      case vt_float://size + float=float
        vt=vt_float;
        v.f+=v.z+rhs_v.v.f;
        return *this;
      case vt_double://size + double=double
        vt=vt_double;
        v.d+=v.z+rhs_v.v.d;
        return *this;
      case vt_ld://size + long double=long double
        vt=vt_ld;
        v.l+=v.z+rhs_v.v.l;
        return *this;
      case vt_int://size + int=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.i;
        return *this;
      case vt_size://size + size=size
        vt=vt_size;
        v.z+=v.z+rhs_v.v.z;
        return *this;
      case vt_string://size + string=size
      case vt_ascii:
        vt=vt_short;
        v.z+=v.s+rhs_v.AsSize();
        return *this;
      }
      return *this;
    case vt_string:
      (*o.s)+=rhs;
      return *this;
    case vt_ascii:
      AsString();
      (*o.s)+=rhs;
      return *this;
    default:
      operator =(rhs);
    }
    return *this;
  }

  //if now change to NOT string/object,then delete last string
  inline void CheckObject(dbool keep_string);
  inline void Ascii2String();//implement in dni.hpp
};

#define MakeStructValue(value,s) value.CheckObject(false);memcpy(value.v.reserved,&s,sizeof(s));value.vt=vt_custom+sizeof(s);

struct dAutoArg
{
  inline dAutoArg()
  {
    count=0;
    del=false;
    memset(values,0,sizeof(values));
  }
  inline dAutoArg(bool self_del)
  {
    count=0;
    del=self_del;
    memset(values,0,sizeof(values));
  }
  inline ~dAutoArg()
  {
    Clear();
  }
  dAutoArg& operator<<(dValue* v)
  {
    values[count++]=v;
    return *this;
  }
  dAutoArg& operator<<(dAutoArg* param)
  {
    if(param)
    {
      for(int i=0;i<param->count;i++)
      {
        values[count++]=(*param)[i];
      }
    }
    return *this;
  }
  inline dValue* operator[](int index)
  {
    return values[index];
  }
  inline void Clear()
  {
    if(del)
    {
      for(int i=0;i<count;i++)
      {
        delete values[i];
      }
    }
    count=0;
  }

  int count;
  bool del;
  dValue *values[32];
};
typedef dAutoArg*        pAutoArg;

struct dField;
struct dMethod
{
  /*all dynamic method have uniform: void f(Method *method,dobject sender,pParam p,dValue *ret_val)*/
  dMethod *     next;
  dchararray    name;
  dchararray    param;
  dint          vt;//return type
  dint          flag;/*0ALL_CALL, 1PRODO_CALL, 2BASE_CALL, 3AFTER_CALL, above 3 other*/
  void *        data[8];//func reserved data
  void *        fp;//function pointer
  dobject       t;//this,target,receiver
  dField *      field;//field pointer
  void *        reserved[7];
};

//0all, 1predo, 2base, 3afterdo, above 3 other
#define ALL_CALL      0
#define PRODO_CALL    1
#define BASE_CALL     2
#define AFTER_CALL    3
#define OTHER_CALL    4

struct dField
{
  /*all dynamic method have uniform: void f(dobject sender,pParam p,dValue *ret_val)*/
  dchararray    name;
  dchararray    td;//type description
  dValue        value;
  dMethod       get_method;//hook mothods
  dMethod       set_method;//hook mothods
  dobject       ext;//extend properties
  void *        data[8];//field reserved data
  void *        reserved[10];
};

//function pointer define
typedef void (__stdcall * DispatchMsgFunc_t)(void *msg);

//dValue pointer wrapper
struct dValuePointerWrapper
{
  dobject obj;
  dint field_index;
  dValue *vp;
  dValue value;

  dValuePointerWrapper():obj(NULL),field_index(0),vp(NULL){}
  dValuePointerWrapper(const dValuePointerWrapper& rhs):obj(rhs.obj),field_index(rhs.field_index),vp(rhs.vp){}

  dValuePointerWrapper(dValue *p):obj(NULL),field_index(0),vp(p){}
  dValuePointerWrapper(const dchararray rhs):obj(NULL),field_index(0),vp(NULL){operator=(L"");}

  inline dValue *GetValuePointer();//declaer in dni.hpp

  //array operator
  inline dchar& operator[](int index)
  {
    static dchar ret[1]={0};
    LWideString *s=NULL;
    s=GetValuePointer()->AsString();
    if(s==NULL)
      return ret[0];
    return s->operator[](index);
  }

  //convert to native operators
  #ifdef __BORLANDC__
  operator AnsiString(){return GetValuePointer()->operator AnsiString();}
  #endif

  inline operator dbool()
  {
    return GetValuePointer()->AsBool();
  }
  inline dbool AsBool(){return operator dbool();}
  inline dValuePointerWrapper& operator =(const dchararray rhs)//set value operator
  {
    (*GetValuePointer())=rhs;
    return *this;
  }
  inline operator dchararray()//get value operator
  {
    return GetValuePointer()->operator dchararray();
  }

  inline dValuePointerWrapper& operator +=(dchar rhs)
  {
    (*GetValuePointer())+=rhs;
    return *this;
  }

  inline dValuePointerWrapper& operator =(const void *rhs)//set value operator
  {
    (*GetValuePointer())=rhs;
    return *this;
  }
};
#define DV_BOOL   dValuePointerWrapper
#define DV_BYTE   dValuePointerWrapper
#define DV_CHAR   dValuePointerWrapper
#define DV_SHORT  dValuePointerWrapper
#define DV_ASCII  dValuePointerWrapper
#define DV_LONG   dValuePointerWrapper
#define DV_SIZE   dValuePointerWrapper
#define DV_FLOAT  dValuePointerWrapper
#define DV_DOUBLE dValuePointerWrapper
#define DV_OBJECT dValuePointerWrapper

struct DV_STRING:public dValuePointerWrapper
{
  DV_STRING():dValuePointerWrapper(L""){}

  DV_STRING(dValue *p):dValuePointerWrapper(p){}
};

//published declear]- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#pragma pack()
#endif //__DTYPES_H__

